@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2015, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -70,7 +71,7 @@ class UserError(Exception):
     """
 
 
-# Utilities.
+# Encoding utilities.
 
 def _out_encoding():
     """Get the encoding to use for *outputting* strings to the console.
@@ -137,6 +138,45 @@ def print_(*strings, **kwargs):
     sys.stdout.write(txt)
 
 
+# Configuration wrappers.
+
+def _bool_fallback(a, b):
+    """Given a boolean or None, return the original value or a fallback.
+    """
+    if a is None:
+        assert isinstance(b, bool)
+        return b
+    else:
+        assert isinstance(a, bool)
+        return a
+
+
+def should_write(write_opt=None):
+    """Decide whether a command that updates metadata should also write
+    tags, using the importer configuration as the default.
+    """
+    return _bool_fallback(write_opt, config['import']['write'].get(bool))
+
+
+def should_move(move_opt=None):
+    """Decide whether a command that updates metadata should also move
+    files when they're inside the library, using the importer
+    configuration as the default.
+
+    Specifically, commands should move files after metadata updates only
+    when the importer is configured *either* to move *or* to copy files.
+    They should avoid moving files when the importer is configured not
+    to touch any filenames.
+    """
+    return _bool_fallback(
+        move_opt,
+        config['import']['move'].get(bool) or
+        config['import']['copy'].get(bool)
+    )
+
+
+# Input prompts.
+
 def input_(prompt=None):
     """Like `raw_input`, but decodes the result to a Unicode string.
     Raises a UserError if stdin is not available. The prompt is sent to
@@ -327,6 +367,37 @@ def input_yn(prompt, require=False):
     return sel == 'y'
 
 
+def input_select_objects(prompt, objs, rep):
+    """Prompt to user to choose all, none, or some of the given objects.
+    Return the list of selected objects.
+
+    `prompt` is the prompt string to use for each question (it should be
+    phrased as an imperative verb). `rep` is a function to call on each
+    object to print it out when confirming objects individually.
+    """
+    choice = input_options(
+        ('y', 'n', 's'), False,
+        '%s? (Yes/no/select)' % prompt)
+    print()  # Blank line.
+
+    if choice == 'y':  # Yes.
+        return objs
+
+    elif choice == 's':  # Select.
+        out = []
+        for obj in objs:
+            rep(obj)
+            if input_yn('%s? (yes/no)' % prompt, True):
+                out.append(obj)
+            print()  # go to a new line
+        return out
+
+    else:  # No.
+        return []
+
+
+# Human output formatting.
+
 def human_bytes(size):
     """Formats size, a number of bytes, in a human-readable way."""
     powers = ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'H']
@@ -374,6 +445,8 @@ def human_seconds_short(interval):
     return u'%i:%02i' % (interval // 60, interval % 60)
 
 
+# Colorization.
+
 # ANSI terminal colorization code heavily inspired by pygments:
 # http://dev.pocoo.org/hg/pygments-main/file/b2deea5b5030/pygments/console.py
 # (pygments is by Tim Hatch, Armin Ronacher, et al.)
@@ -1128,11 +1201,11 @@ def _raw_main(args, lib=None):
     parser.add_option('-d', '--directory', dest='directory',
                       help="destination music directory")
     parser.add_option('-v', '--verbose', dest='verbose', action='count',
-                      help='print debugging information')
+                      help='log more details (use twice for even more)')
     parser.add_option('-c', '--config', dest='config',
                       help='path to configuration file')
     parser.add_option('-h', '--help', dest='help', action='store_true',
-                      help='how this help message and exit')
+                      help='show this help message and exit')
     parser.add_option('--version', dest='version', action='store_true',
                       help=optparse.SUPPRESS_HELP)
 
