@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2015, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -21,6 +22,8 @@ from __future__ import (division, absolute_import, print_function,
 
 import os
 import re
+from collections import namedtuple, Counter
+from itertools import chain
 
 import beets
 from beets import ui
@@ -38,6 +41,7 @@ from beets import logging
 from beets.util.confit import _package_path
 
 VARIOUS_ARTISTS = u'Various Artists'
+PromptChoice = namedtuple('ExtraChoice', ['short', 'long', 'callback'])
 
 # Global logger.
 log = logging.getLogger('beets')
@@ -77,6 +81,14 @@ def _do_query(lib, query, album, also_items=True):
 
 # fields: Shows a list of available fields for queries and format strings.
 
+def _print_keys(query):
+    """Given a SQLite query result, print the `key` field of each
+    returned row, with identation of 2 spaces.
+    """
+    for row in query:
+        print_(' ' * 2 + row[b'key'])
+
+
 def fields_func(lib, opts, args):
     def _print_rows(names):
         names.sort()
@@ -88,6 +100,15 @@ def fields_func(lib, opts, args):
     print_("Album fields:")
     _print_rows(library.Album.all_keys())
 
+    with lib.transaction() as tx:
+        # The SQL uses the DISTINCT to get unique values from the query
+        unique_fields = 'SELECT DISTINCT key FROM (%s)'
+
+        print_("Item flexible attributes:")
+        _print_keys(tx.query(unique_fields % library.Item._flex_table))
+
+        print_("Album flexible attributes:")
+        _print_keys(tx.query(unique_fields % library.Album._flex_table))
 
 fields_cmd = ui.Subcommand(
     'fields',
@@ -436,7 +457,7 @@ def summarize_items(items, singleton):
     return ', '.join(summary_parts)
 
 
-def _summary_judment(rec):
+def _summary_judgment(rec):
     """Determines whether a decision should be made without even asking
     the user. This occurs in quiet mode and when an action is chosen for
     NONE recommendations. Return an action or None if the user should be
@@ -470,7 +491,8 @@ def _summary_judment(rec):
 
 
 def choose_candidate(candidates, singleton, rec, cur_artist=None,
-                     cur_album=None, item=None, itemcount=None):
+                     cur_album=None, item=None, itemcount=None,
+                     extra_choices=[]):
     """Given a sorted list of candidates, ask the user for a selection
     of which candidate to use. Applies to both full albums and
     singletons  (tracks). Candidates are either AlbumMatch or TrackMatch
@@ -478,8 +500,16 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
     `cur_album`, and `itemcount` must be provided. For singletons,
     `item` must be provided.
 
-    Returns the result of the choice, which may SKIP, ASIS, TRACKS, or
-    MANUAL or a candidate (an AlbumMatch/TrackMatch object).
+    `extra_choices` is a list of `PromptChoice`s, containg the choices
+    appended by the plugins after receiving the `before_choose_candidate`
+    event. If not empty, the choices are appended to the prompt presented
+    to the user.
+
+    Returns one of the following:
+    * the result of the choice, which may be SKIP, ASIS, TRACKS, or MANUAL
+    * a candidate (an AlbumMatch/TrackMatch object)
+    * the short letter of a `PromptChoice` (if the user selected one of
+    the `extra_choices`).
     """
     # Sanity check.
     if singleton:
@@ -488,6 +518,10 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
         assert cur_artist is not None
         assert cur_album is not None
 
+    # Build helper variables for extra choices.
+    extra_opts = tuple(c.long for c in extra_choices)
+    extra_actions = tuple(c.short for c in extra_choices)
+
     # Zero candidates.
     if not candidates:
         if singleton:
@@ -501,7 +535,7 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
                    'http://beets.readthedocs.org/en/latest/faq.html#nomatch')
             opts = ('Use as-is', 'as Tracks', 'Group albums', 'Skip',
                     'Enter search', 'enter Id', 'aBort')
-        sel = ui.input_options(opts)
+        sel = ui.input_options(opts + extra_opts)
         if sel == 'u':
             return importer.action.ASIS
         elif sel == 't':
@@ -517,6 +551,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
             return importer.action.MANUAL_ID
         elif sel == 'g':
             return importer.action.ALBUMS
+        elif sel in extra_actions:
+            return sel
         else:
             assert False
 
@@ -570,7 +606,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
             else:
                 opts = ('Skip', 'Use as-is', 'as Tracks', 'Group albums',
                         'Enter search', 'enter Id', 'aBort')
-            sel = ui.input_options(opts, numrange=(1, len(candidates)))
+            sel = ui.input_options(opts + extra_opts,
+                                   numrange=(1, len(candidates)))
             if sel == 's':
                 return importer.action.SKIP
             elif sel == 'u':
@@ -588,6 +625,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
                 return importer.action.MANUAL_ID
             elif sel == 'g':
                 return importer.action.ALBUMS
+            elif sel in extra_actions:
+                return sel
             else:  # Numerical selection.
                 match = candidates[sel - 1]
                 if sel != 1:
@@ -622,7 +661,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
         })
         if default is None:
             require = True
-        sel = ui.input_options(opts, require=require, default=default)
+        sel = ui.input_options(opts + extra_opts, require=require,
+                               default=default)
         if sel == 'a':
             return match
         elif sel == 'g':
@@ -640,6 +680,8 @@ def choose_candidate(candidates, singleton, rec, cur_artist=None,
             raise importer.ImportAbort()
         elif sel == 'i':
             return importer.action.MANUAL_ID
+        elif sel in extra_actions:
+            return sel
 
 
 def manual_search(singleton):
@@ -672,7 +714,7 @@ class TerminalImportSession(importer.ImportSession):
                u' ({0} items)'.format(len(task.items)))
 
         # Take immediate action if appropriate.
-        action = _summary_judment(task.rec)
+        action = _summary_judgment(task.rec)
         if action == importer.action.APPLY:
             match = task.candidates[0]
             show_change(task.cur_artist, task.cur_album, match)
@@ -683,10 +725,14 @@ class TerminalImportSession(importer.ImportSession):
         # Loop until we have a choice.
         candidates, rec = task.candidates, task.rec
         while True:
+            # Gather extra choices from plugins.
+            extra_choices = self._get_plugin_choices(task)
+            extra_ops = {c.short: c.callback for c in extra_choices}
+
             # Ask for a choice from the user.
             choice = choose_candidate(
                 candidates, False, rec, task.cur_artist, task.cur_album,
-                itemcount=len(task.items)
+                itemcount=len(task.items), extra_choices=extra_choices
             )
 
             # Choose which tags to use.
@@ -705,8 +751,14 @@ class TerminalImportSession(importer.ImportSession):
                 search_id = manual_id(False)
                 if search_id:
                     _, _, candidates, rec = autotag.tag_album(
-                        task.items, search_id=search_id
+                        task.items, search_ids=search_id.split()
                     )
+            elif choice in extra_ops.keys():
+                # Allow extra ops to automatically set the post-choice.
+                post_choice = extra_ops[choice](self, task)
+                if isinstance(post_choice, importer.action):
+                    # MANUAL and MANUAL_ID have no effect, even if returned.
+                    return post_choice
             else:
                 # We have a candidate! Finish tagging. Here, choice is an
                 # AlbumMatch object.
@@ -722,7 +774,7 @@ class TerminalImportSession(importer.ImportSession):
         candidates, rec = task.candidates, task.rec
 
         # Take immediate action if appropriate.
-        action = _summary_judment(task.rec)
+        action = _summary_judgment(task.rec)
         if action == importer.action.APPLY:
             match = candidates[0]
             show_item_change(task.item, match)
@@ -731,8 +783,12 @@ class TerminalImportSession(importer.ImportSession):
             return action
 
         while True:
+            extra_choices = self._get_plugin_choices(task)
+            extra_ops = {c.short: c.callback for c in extra_choices}
+
             # Ask for a choice.
-            choice = choose_candidate(candidates, True, rec, item=task.item)
+            choice = choose_candidate(candidates, True, rec, item=task.item,
+                                      extra_choices=extra_choices)
 
             if choice in (importer.action.SKIP, importer.action.ASIS):
                 return choice
@@ -747,8 +803,14 @@ class TerminalImportSession(importer.ImportSession):
                 # Ask for a track ID.
                 search_id = manual_id(True)
                 if search_id:
-                    candidates, rec = autotag.tag_item(task.item,
-                                                       search_id=search_id)
+                    candidates, rec = autotag.tag_item(
+                        task.item, search_ids=search_id.split())
+            elif choice in extra_ops.keys():
+                # Allow extra ops to automatically set the post-choice.
+                post_choice = extra_ops[choice](self, task)
+                if isinstance(post_choice, importer.action):
+                    # MANUAL and MANUAL_ID have no effect, even if returned.
+                    return post_choice
             else:
                 # Chose a candidate.
                 assert isinstance(choice, autotag.TrackMatch)
@@ -761,16 +823,6 @@ class TerminalImportSession(importer.ImportSession):
         log.warn(u"This {0} is already in the library!",
                  ("album" if task.is_album else "item"))
 
-        # skip empty albums (coming from a previous failed import session)
-        if task.is_album:
-            real_duplicates = [dup for dup in found_duplicates if dup.items()]
-            if not real_duplicates:
-                log.info("All duplicates are empty, we ignore them")
-                task.should_remove_duplicates = True
-                return
-        else:
-            real_duplicates = found_duplicates
-
         if config['import']['quiet']:
             # In quiet mode, don't prompt -- just skip.
             log.info(u'Skipping.')
@@ -778,17 +830,12 @@ class TerminalImportSession(importer.ImportSession):
         else:
             # Print some detail about the existing and new items so the
             # user can make an informed decision.
-            for duplicate in real_duplicates:
+            for duplicate in found_duplicates:
                 print_("Old: " + summarize_items(
                     list(duplicate.items()) if task.is_album else [duplicate],
                     not task.is_album,
                 ))
 
-            if real_duplicates != found_duplicates:  # there's empty albums
-                count = len(found_duplicates) - len(real_duplicates)
-                print_("Old: {0} empty album{1}".format(
-                       count, "s" if count > 1 else ""))
-
             print_("New: " + summarize_items(
                 task.imported_items(),
                 not task.is_album,
@@ -815,6 +862,50 @@ class TerminalImportSession(importer.ImportSession):
                            "was interrupted. Resume (Y/n)?"
                            .format(displayable_path(path)))
 
+    def _get_plugin_choices(self, task):
+        """Get the extra choices appended to the plugins to the ui prompt.
+
+        The `before_choose_candidate` event is sent to the plugins, with
+        session and task as its parameters. Plugins are responsible for
+        checking the right conditions and returning a list of `PromptChoice`s,
+        which is flattened and checked for conflicts.
+
+        If two or more choices have the same short letter, a warning is
+        emitted and all but one choices are discarded, giving preference
+        to the default importer choices.
+
+        Returns a list of `PromptChoice`s.
+        """
+        # Send the before_choose_candidate event and flatten list.
+        extra_choices = list(chain(*plugins.send('before_choose_candidate',
+                                                 session=self, task=task)))
+        # Take into account default options, for duplicate checking.
+        all_choices = [PromptChoice('a', 'Apply', None),
+                       PromptChoice('s', 'Skip', None),
+                       PromptChoice('u', 'Use as-is', None),
+                       PromptChoice('t', 'as Tracks', None),
+                       PromptChoice('g', 'Group albums', None),
+                       PromptChoice('e', 'Enter search', None),
+                       PromptChoice('i', 'enter Id', None),
+                       PromptChoice('b', 'aBort', None)] +\
+            extra_choices
+
+        short_letters = [c.short for c in all_choices]
+        if len(short_letters) != len(set(short_letters)):
+            # Duplicate short letter has been found.
+            duplicates = [i for i, count in Counter(short_letters).items()
+                          if count > 1]
+            for short in duplicates:
+                # Keep the first of the choices, removing the rest.
+                dup_choices = [c for c in all_choices if c.short == short]
+                for c in dup_choices[1:]:
+                    log.warn(u"Prompt choice '{0}' removed due to conflict "
+                             u"with '{1}' (short letter: '{2}')",
+                             c.long, dup_choices[0].long, c.short)
+                    extra_choices.remove(c)
+        return extra_choices
+
+
 # The import command.
 
 
@@ -950,6 +1041,11 @@ import_cmd.parser.add_option(
     '--pretend', dest='pretend', action='store_true',
     help='just print the files to import'
 )
+import_cmd.parser.add_option(
+    '-S', '--search-id', dest='search_ids', action='append',
+    metavar='BACKEND_ID',
+    help='restrict matching to a specific metadata backend ID'
+)
 import_cmd.func = import_func
 default_commands.append(import_cmd)
 
@@ -1070,7 +1166,8 @@ def update_items(lib, query, album, move, pretend):
 
 
 def update_func(lib, opts, args):
-    update_items(lib, decargs(args), opts.album, opts.move, opts.pretend)
+    update_items(lib, decargs(args), opts.album, ui.should_move(opts.move),
+                 opts.pretend)
 
 
 update_cmd = ui.Subcommand(
@@ -1079,7 +1176,11 @@ update_cmd = ui.Subcommand(
 update_cmd.parser.add_album_option()
 update_cmd.parser.add_format_option()
 update_cmd.parser.add_option(
-    '-M', '--nomove', action='store_false', default=True, dest='move',
+    '-m', '--move', action='store_true', dest='move',
+    help="move files in the library directory"
+)
+update_cmd.parser.add_option(
+    '-M', '--nomove', action='store_false', dest='move',
     help="don't move files in library"
 )
 update_cmd.parser.add_option(
@@ -1155,7 +1256,10 @@ def show_stats(lib, query, exact):
 
     for item in items:
         if exact:
-            total_size += os.path.getsize(item.path)
+            try:
+                total_size += os.path.getsize(item.path)
+            except OSError as exc:
+                log.info('could not get size of {}: {}', item.path, exc)
         else:
             total_size += int(item.length * item.bitrate / 8)
         total_time += item.length
@@ -1245,13 +1349,7 @@ def modify_items(lib, mods, dels, query, write, move, album, confirm):
            .format(len(objs), 'album' if album else 'item'))
     changed = set()
     for obj in objs:
-        obj.update(mods)
-        for field in dels:
-            try:
-                del obj[field]
-            except KeyError:
-                pass
-        if ui.show_model_changes(obj):
+        if print_and_modify(obj, mods, dels):
             changed.add(obj)
 
     # Still something to do?
@@ -1270,19 +1368,31 @@ def modify_items(lib, mods, dels, query, write, move, album, confirm):
         else:
             extra = ''
 
-        if not ui.input_yn('Really modify%s (Y/n)?' % extra):
-            return
+        changed = ui.input_select_objects(
+            'Really modify%s' % extra, changed,
+            lambda o: print_and_modify(o, mods, dels)
+        )
 
     # Apply changes to database and files
     with lib.transaction():
         for obj in changed:
-            if move:
-                cur_path = obj.path
-                if lib.directory in ancestry(cur_path):  # In library?
-                    log.debug(u'moving object {0}', displayable_path(cur_path))
-                    obj.move()
+            obj.try_sync(write, move)
 
-            obj.try_sync(write)
+
+def print_and_modify(obj, mods, dels):
+    """Print the modifications to an item and return a bool indicating
+    whether any changes were made.
+
+    `mods` is a dictionary of fields and values to update on the object;
+    `dels` is a sequence of fields to delete.
+    """
+    obj.update(mods)
+    for field in dels:
+        try:
+            del obj[field]
+        except KeyError:
+            pass
+    return ui.show_model_changes(obj)
 
 
 def modify_parse_args(args):
@@ -1308,17 +1418,19 @@ def modify_func(lib, opts, args):
     query, mods, dels = modify_parse_args(decargs(args))
     if not mods and not dels:
         raise ui.UserError('no modifications specified')
-    write = opts.write if opts.write is not None else \
-        config['import']['write'].get(bool)
-    modify_items(lib, mods, dels, query, write, opts.move, opts.album,
-                 not opts.yes)
+    modify_items(lib, mods, dels, query, ui.should_write(opts.write),
+                 ui.should_move(opts.move), opts.album, not opts.yes)
 
 
 modify_cmd = ui.Subcommand(
     'modify', help='change metadata fields', aliases=('mod',)
 )
 modify_cmd.parser.add_option(
-    '-M', '--nomove', action='store_false', default=True, dest='move',
+    '-m', '--move', action='store_true', dest='move',
+    help="move files in the library directory"
+)
+modify_cmd.parser.add_option(
+    '-M', '--nomove', action='store_false', dest='move',
     help="don't move files in library"
 )
 modify_cmd.parser.add_option(
@@ -1341,7 +1453,7 @@ default_commands.append(modify_cmd)
 
 # move: Move/copy files to the library or a new base directory.
 
-def move_items(lib, dest, query, copy, album, pretend):
+def move_items(lib, dest, query, copy, album, pretend, confirm=False):
     """Moves or copies items to a new base directory, given by dest. If
     dest is None, then the library's base directory is used, making the
     command "consolidate" files.
@@ -1349,10 +1461,19 @@ def move_items(lib, dest, query, copy, album, pretend):
     items, albums = _do_query(lib, query, album, False)
     objs = albums if album else items
 
+    # Filter out files that don't need to be moved.
+    isitemmoved = lambda item: item.path != item.destination(basedir=dest)
+    isalbummoved = lambda album: any(isitemmoved(i) for i in album.items())
+    objs = [o for o in objs if (isalbummoved if album else isitemmoved)(o)]
+
     action = 'Copying' if copy else 'Moving'
+    act = 'copy' if copy else 'move'
     entity = 'album' if album else 'item'
     log.info(u'{0} {1} {2}{3}.', action, len(objs), entity,
-             's' if len(objs) > 1 else '')
+             's' if len(objs) != 1 else '')
+    if not objs:
+        return
+
     if pretend:
         if album:
             show_path_changes([(item.path, item.destination(basedir=dest))
@@ -1361,6 +1482,12 @@ def move_items(lib, dest, query, copy, album, pretend):
             show_path_changes([(obj.path, obj.destination(basedir=dest))
                                for obj in objs])
     else:
+        if confirm:
+            objs = ui.input_select_objects(
+                'Really %s' % act, objs,
+                lambda o: show_path_changes(
+                    [(o.path, o.destination(basedir=dest))]))
+
         for obj in objs:
             log.debug(u'moving: {0}', util.displayable_path(obj.path))
 
@@ -1375,7 +1502,8 @@ def move_func(lib, opts, args):
         if not os.path.isdir(dest):
             raise ui.UserError('no such directory: %s' % dest)
 
-    move_items(lib, dest, decargs(args), opts.copy, opts.album, opts.pretend)
+    move_items(lib, dest, decargs(args), opts.copy, opts.album, opts.pretend,
+               opts.timid)
 
 
 move_cmd = ui.Subcommand(
@@ -1391,7 +1519,12 @@ move_cmd.parser.add_option(
 )
 move_cmd.parser.add_option(
     '-p', '--pretend', default=False, action='store_true',
-    help='show how files would be moved, but don\'t touch anything')
+    help='show how files would be moved, but don\'t touch anything'
+)
+move_cmd.parser.add_option(
+    '-t', '--timid', dest='timid', action='store_true',
+    help='always confirm all actions'
+)
 move_cmd.parser.add_album_option()
 move_cmd.func = move_func
 default_commands.append(move_cmd)
@@ -1423,7 +1556,9 @@ def write_items(lib, query, pretend, force):
         changed = ui.show_model_changes(item, clean_item,
                                         library.Item._media_tag_fields, force)
         if (changed or force) and not pretend:
-            item.try_sync()
+            # We use `try_sync` here to keep the mtime up to date in the
+            # database.
+            item.try_sync(True, False)
 
 
 def write_func(lib, opts, args):
@@ -1481,7 +1616,7 @@ def config_edit():
     An empty config file is created if no existing config file exists.
     """
     path = config.user_config_path()
-    editor = os.environ.get('EDITOR')
+    editor = util.editor_command()
     try:
         if not os.path.isfile(path):
             open(path, 'w+').close()
