@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2015, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -776,16 +776,19 @@ def interactive_open(targets, command):
 
     Can raise `OSError`.
     """
+    assert command
+
     # Split the command string into its arguments.
     try:
-        command = shlex_split(command)
+        args = shlex_split(command)
     except ValueError:  # Malformed shell tokens.
-        command = [command]
-    command.insert(0, command[0])  # for argv[0]
+        args = [command]
 
-    command += targets
+    args.insert(0, args[0])  # for argv[0]
 
-    return os.execlp(*command)
+    args += targets
+
+    return os.execlp(*args)
 
 
 def _windows_long_path_name(short_path):
