@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2015, Adrian Sampson.
+# Copyright 2016, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -119,6 +119,25 @@ def print_data(data, item=None, fmt=None):
         ui.print_(lineformat.format(field, value))
 
 
+def print_data_keys(data, item=None):
+    """Print only the keys (field names) for an item.
+    """
+    path = displayable_path(item.path) if item else None
+    formatted = []
+    for key, value in data.iteritems():
+        formatted.append(key)
+
+    if len(formatted) == 0:
+        return
+
+    line_format = u'{0}{{0}}'.format(u' ' * 4)
+    if path:
+        ui.print_(displayable_path(path))
+
+    for field in sorted(formatted):
+        ui.print_(line_format.format(field))
+
+
 class InfoPlugin(BeetsPlugin):
 
     def commands(self):
@@ -131,6 +150,8 @@ class InfoPlugin(BeetsPlugin):
         cmd.parser.add_option('-i', '--include-keys', default=[],
                               action='append', dest='included_keys',
                               help='comma separated list of keys to show')
+        cmd.parser.add_option('-k', '--keys-only', action='store_true',
+                              help='show only the keys')
         cmd.parser.add_format_option(target='item')
         return [cmd]
 
@@ -173,7 +194,10 @@ class InfoPlugin(BeetsPlugin):
             else:
                 if not first:
                     ui.print_()
-                print_data(data, item, opts.format)
+                if opts.keys_only:
+                    print_data_keys(data, item)
+                else:
+                    print_data(data, item, opts.format)
                 first = False
 
         if opts.summarize:
