@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 # This file is part of beets.
-# Copyright 2015, Fabrice Laporte, Yevgeny Bezman, and Adrian Sampson.
+# Copyright 2016, Fabrice Laporte, Yevgeny Bezman, and Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -27,7 +28,6 @@ from beets import logging
 from beets import ui
 from beets.plugins import BeetsPlugin
 from beets.util import syspath, command_output, displayable_path
-from beets import config
 
 
 # Utilities.
@@ -43,6 +43,11 @@ class FatalReplayGainError(Exception):
     """
 
 
+class FatalGstreamerPluginReplayGainError(FatalReplayGainError):
+    """Raised when a fatal error occurs in the GStreamerBackend when
+    loading the required plugins."""
+
+
 def call(args):
     """Execute the command and return its output or raise a
     ReplayGainError on failure.
@@ -317,6 +322,7 @@ class CommandBackend(Backend):
         the album gain
         """
         if len(items) == 0:
+            self._log.debug('no supported tracks to analyze')
             return []
 
         """Compute ReplayGain values and return a list of results
@@ -390,7 +396,7 @@ class GStreamerBackend(Backend):
 
         if self._src is None or self._decbin is None or self._conv is None \
            or self._res is None or self._rg is None:
-            raise FatalReplayGainError(
+            raise FatalGstreamerPluginReplayGainError(
                 "Failed to load required GStreamer plugins"
             )
 
@@ -495,14 +501,25 @@ class GStreamerBackend(Backend):
         if len(self._file_tags) != len(items):
             raise ReplayGainError("Some items in album did not receive tags")
 
-        ret = []
+        # Collect track gains.
+        track_gains = []
         for item in items:
-            ret.append(Gain(self._file_tags[item]["TRACK_GAIN"],
-                            self._file_tags[item]["TRACK_PEAK"]))
+            try:
+                gain = self._file_tags[item]["TRACK_GAIN"]
+                peak = self._file_tags[item]["TRACK_PEAK"]
+            except KeyError:
+                raise ReplayGainError("results missing for track")
+            track_gains.append(Gain(gain, peak))
 
+        # Get album gain information from the last track.
         last_tags = self._file_tags[items[-1]]
-        return AlbumGain(Gain(last_tags["ALBUM_GAIN"],
-                              last_tags["ALBUM_PEAK"]), ret)
+        try:
+            gain = last_tags["ALBUM_GAIN"]
+            peak = last_tags["ALBUM_PEAK"]
+        except KeyError:
+            raise ReplayGainError("results missing for album")
+
+        return AlbumGain(Gain(gain, peak), track_gains)
 
     def close(self):
         self._bus.remove_signal_watch()
@@ -521,10 +538,9 @@ class GStreamerBackend(Backend):
         err, debug = message.parse_error()
         f = self._src.get_property("location")
         # A GStreamer error, either an unsupported format or a bug.
-        self._error = \
-            ReplayGainError(u"Error {0} - {1} on file {2}".format(err,
-                                                                  debug,
-                                                                  f))
+        self._error = ReplayGainError(
+            "Error {0!r} - {1!r} on file {2!r}".format(err, debug, f)
+        )
 
     def _on_tag(self, bus, message):
         tags = message.parse_tag()
@@ -727,7 +743,7 @@ class AudioToolsBackend(Backend):
 
         # Each call to title_gain on a ReplayGain object returns peak and gain
         # of the track.
-        rg_track_gain, rg_track_peak = rg._title_gain(rg, audiofile)
+        rg_track_gain, rg_track_peak = self._title_gain(rg, audiofile)
 
         self._log.debug(u'ReplayGain for track {0} - {1}: {2:.2f}, {3:.2f}',
                         item.artist, item.title, rg_track_gain, rg_track_peak)
@@ -925,7 +941,7 @@ class ReplayGainPlugin(BeetsPlugin):
         def func(lib, opts, args):
             self._log.setLevel(logging.INFO)
 
-            write = config['import']['write'].get(bool)
+            write = ui.should_write()
 
             if opts.album:
                 for album in lib.albums(ui.decargs(args)):
